@@ -4,15 +4,18 @@ pub mod primitives;
 pub mod state;
 
 use crate::components::core::{image::Handle, shapes::Rectangle};
+use crate::components::filters::FiltersBrush;
 use crate::components::layer::{self, LayerBrush};
 use crate::components::quad::QuadBrush;
 use crate::components::rect::{Rect, RectBrush};
 use crate::components::rich_text::RichTextBrush;
 use crate::components::text;
 use crate::font::{fonts::SugarloafFont, FontLibrary};
-use crate::layout::SugarloafLayout;
+use crate::layout::{RichTextLayout, RootStyle};
 use crate::sugarloaf::graphics::{BottomLayer, Graphics};
 use crate::sugarloaf::layer::types;
+use crate::Content;
+use crate::SugarDimensions;
 use crate::{context::Context, Object};
 use ab_glyph::{self, PxScale};
 use core::fmt::{Debug, Formatter};
@@ -33,6 +36,7 @@ pub struct Sugarloaf<'a> {
     pub background_color: Option<wgpu::Color>,
     pub background_image: Option<ImageProperties>,
     pub graphics: Graphics,
+    filters_brush: FiltersBrush,
 }
 
 #[derive(Debug)]
@@ -117,7 +121,7 @@ impl Sugarloaf<'_> {
         window: SugarloafWindow,
         renderer: SugarloafRenderer,
         font_library: &FontLibrary,
-        layout: SugarloafLayout,
+        layout: RootStyle,
     ) -> Result<Sugarloaf<'a>, SugarloafWithErrors<'a>> {
         let font_features = renderer.font_features.to_owned();
         let ctx = Context::new(window, renderer);
@@ -133,6 +137,7 @@ impl Sugarloaf<'_> {
         let quad_brush = QuadBrush::new(&ctx);
         let rich_text_brush = RichTextBrush::new(&ctx);
         let state = SugarState::new(layout, font_library, &font_features);
+        let filters_brush = FiltersBrush::default();
 
         let instance = Sugarloaf {
             state,
@@ -145,6 +150,7 @@ impl Sugarloaf<'_> {
             rich_text_brush,
             text_brush,
             graphics: Graphics::default(),
+            filters_brush,
         };
 
         Ok(instance)
@@ -154,9 +160,9 @@ impl Sugarloaf<'_> {
     pub fn update_font(&mut self, font_library: &FontLibrary) {
         tracing::info!("requested a font change");
 
-        self.rich_text_brush.reset();
         self.state.reset_compositors();
-        self.state.set_fonts(font_library);
+        self.state
+            .set_fonts(font_library, &mut self.rich_text_brush);
     }
 
     #[inline]
@@ -170,18 +176,37 @@ impl Sugarloaf<'_> {
     }
 
     #[inline]
-    pub fn layout(&self) -> SugarloafLayout {
-        self.state.layout
+    pub fn style(&self) -> RootStyle {
+        self.state.style
     }
 
     #[inline]
-    pub fn layout_mut(&mut self) -> &mut SugarloafLayout {
-        &mut self.state.layout
+    pub fn style_mut(&mut self) -> &mut RootStyle {
+        &mut self.state.style
     }
 
     #[inline]
-    pub fn update_font_size(&mut self, operation: u8) {
-        self.state.compute_layout_font_size(operation);
+    pub fn set_rich_text_font_size_based_on_action(
+        &mut self,
+        rt_id: &usize,
+        operation: u8,
+    ) {
+        self.state.set_rich_text_font_size_based_on_action(
+            rt_id,
+            operation,
+            &mut self.rich_text_brush,
+        );
+    }
+
+    #[inline]
+    pub fn set_rich_text_font_size(&mut self, rt_id: &usize, font_size: f32) {
+        self.state
+            .set_rich_text_font_size(rt_id, font_size, &mut self.rich_text_brush);
+    }
+
+    #[inline]
+    pub fn update_filters(&mut self, filter_paths: &[String]) {
+        self.filters_brush.update_filters(&self.ctx, filter_paths);
     }
 
     #[inline]
@@ -209,7 +234,16 @@ impl Sugarloaf<'_> {
     }
 
     #[inline]
-    pub fn content(&mut self) -> &mut crate::Content {
+    pub fn create_rich_text(&mut self) -> usize {
+        self.state.create_rich_text()
+    }
+
+    #[inline]
+    pub fn clear_rich_text(&mut self, id: &usize) {
+        self.state.clear_rich_text(id);
+    }
+
+    pub fn content(&mut self) -> &mut Content {
         self.state.content()
     }
 
@@ -218,15 +252,35 @@ impl Sugarloaf<'_> {
         self.state.compute_objects(objects);
     }
 
+    #[inline]
+    pub fn rich_text_layout(&self, id: &usize) -> RichTextLayout {
+        self.state.get_state_layout(id)
+    }
+
+    #[inline]
+    pub fn get_rich_text_dimensions(&mut self, id: &usize) -> SugarDimensions {
+        self.state
+            .get_rich_text_dimensions(id, &mut self.rich_text_brush)
+    }
+
     #[inline]
     pub fn clear(&mut self) {
         self.state.clean_screen();
     }
 
+    #[inline]
+    pub fn window_size(&self) -> SugarloafWindowSize {
+        self.ctx.size
+    }
+
+    #[inline]
+    pub fn scale_factor(&self) -> f32 {
+        self.state.style.scale_factor
+    }
+
     #[inline]
     pub fn resize(&mut self, width: u32, height: u32) {
         self.ctx.resize(width, height);
-        self.state.compute_layout_resize(width, height);
         if let Some(bottom_layer) = &mut self.graphics.bottom_layer {
             if bottom_layer.should_fit {
                 bottom_layer.data.bounds.width = self.ctx.size.width;
@@ -238,7 +292,8 @@ impl Sugarloaf<'_> {
     #[inline]
     pub fn rescale(&mut self, scale: f32) {
         self.ctx.scale = scale;
-        self.state.compute_layout_rescale(scale);
+        self.state
+            .compute_layout_rescale(scale, &mut self.rich_text_brush);
         if let Some(bottom_layer) = &mut self.graphics.bottom_layer {
             if bottom_layer.should_fit {
                 bottom_layer.data.bounds.width = self.ctx.size.width;
@@ -254,7 +309,6 @@ impl Sugarloaf<'_> {
 
     #[inline]
     pub fn render(&mut self) {
-        self.state.compute_changes();
         self.state.compute_dimensions(&mut self.rich_text_brush);
 
         self.state.compute_updates(
@@ -272,7 +326,7 @@ impl Sugarloaf<'_> {
                     &wgpu::CommandEncoderDescriptor { label: None },
                 );
 
-                let view = &frame
+                let view = frame
                     .texture
                     .create_view(&wgpu::TextureViewDescriptor::default());
 
@@ -312,7 +366,7 @@ impl Sugarloaf<'_> {
                             occlusion_query_set: None,
                             label: None,
                             color_attachments: &[Some(wgpu::RenderPassColorAttachment {
-                                view,
+                                view: &view,
                                 resolve_target: None,
                                 ops: wgpu::Operations {
                                     load,
@@ -337,8 +391,7 @@ impl Sugarloaf<'_> {
                         }
                     }
 
-                    self.rich_text_brush
-                        .render(&mut self.ctx, &self.state, &mut rpass);
+                    self.rich_text_brush.render(&mut self.ctx, &mut rpass);
 
                     self.quad_brush
                         .render(&mut self.ctx, &self.state, &mut rpass);
@@ -356,6 +409,13 @@ impl Sugarloaf<'_> {
                     self.graphics.clear_top_layer();
                 }
 
+                self.filters_brush.render(
+                    &self.ctx,
+                    &mut encoder,
+                    &frame.texture,
+                    &frame.texture,
+                );
+
                 self.ctx.queue.submit(Some(encoder.finish()));
                 frame.present();
             }
